'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol ? "symbol" : typeof obj; };

exports.generate = generate;

var _chain = require('./templateParsers/chain.js');

var _chain2 = _interopRequireDefault(_chain);

var _factory = require('./templateParsers/factory.js');

var _factory2 = _interopRequireDefault(_factory);

var _action = require('./templateParsers/action.js');

var _action2 = _interopRequireDefault(_action);

var _signal = require('./templateParsers/signal.js');

var _signal2 = _interopRequireDefault(_signal);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function generateContent(part, partType, config) {
    var content = void 0;
    switch (partType) {
        case 'chainFactory':
            content = (0, _factory2.default)(part, config.templates.chainFactory);
            break;
        case 'actionFactory':
            content = (0, _factory2.default)(part, config.templates.actionFactory);
            break;
        case 'action':
            content = (0, _action2.default)(part, config.templates.action);
            break;
        case 'chain':
            content = (0, _chain2.default)(part, config.templates.chain);
            break;
        case 'signal':
            content = (0, _signal2.default)(part, config.templates.signal);
            break;
        default:
            content = '';
    }

    return content;
}

function generateImports(parseResult, config) {
    var typePaths = config.paths;


    return parseResult.newParts.map(function (part) {
        var line = '';
        var path = '';
        var shouldCreateFile = false;

        var partType = part.type;
        if (partType === 'chain_signal') {
            partType = config.legacy.signalFiles ? 'signal' : 'chain';
        }

        if (config.specialImportsMap[part.identifier]) {
            path = config.specialImportsMap[part.identifier];
        } else {
            path = typePaths[partType] + '/' + part.identifier + '.js';
            shouldCreateFile = true;

            if (parseResult.type === 'SignalOrChainFile') {
                path = '../' + path;
            } else {
                path = './' + path;
            }
        }

        line = 'import ' + part.identifier + ' from \'' + path + '\'' + (config.style.imports.semiColon ? ';' : '');

        var content = false;
        if (shouldCreateFile) {
            content = generateContent(part, partType, config);
            content = content.replace(/\ {2}/g, config.style.indentation);
        }

        return {
            identifiers: [part.identifier],
            lines: [line],
            path: path,
            shouldCreateFile: shouldCreateFile,
            content: content
        };
    });
}

function generate(fileContent, parseResult, config) {
    var newImports = generateImports(parseResult, config);

    var newContent = fileContent;
    var imports = newImports;

    if (config.style.imports.sortAll) {
        (function () {
            var lines = fileContent.split('\n');
            parseResult.imports.map(function (imp) {
                for (var i = imp.loc.start; i < imp.loc.end; i++) {
                    lines[i] = '_____REMOVE_THIS_LINE_____';
                }
            });
            newContent = lines.filter(function (line) {
                return line !== '_____REMOVE_THIS_LINE_____';
            }).join('\n');

            imports = [].concat(_toConsumableArray(parseResult.imports), _toConsumableArray(newImports));
        })();
    }

    if (config.style.imports.sortBy === 'type') {
        imports = imports.sort(function (a, b) {
            if (a.path.slice(0, 1) === '.' && b.path.slice(0, 1) !== '.') {
                return 1;
            }

            if (b.path.slice(0, 1) === '.' && a.path.slice(0, 1) !== '.') {
                return -1;
            }

            if (a.path > b.path) {
                return 1;
            }

            if (a.path < b.path) {
                return -1;
            }

            return 0;
        });
    }

    if (imports && imports.length > 0) {
        var _ret2 = function () {
            var groupRegex = '(' + Object.keys(config.paths).reduce(function (sum, key) {
                return [].concat(_toConsumableArray(sum), [config.paths[key]]);
            }, []).join('|') + ')';

            var importsText = '';
            var lastFirstPartOfPath = false;
            imports.map(function (imp) {
                var matches = imp.path.match(groupRegex);
                var thisFirstPartOfPath = matches && matches.length ? matches[0] : imp.path.slice(0, 6);
                if (config.style.imports.separateGroups !== 'none' && (config.style.imports.separateGroups === 'all' || config.style.imports.separateGroups === 'local' && thisFirstPartOfPath.slice(0, 1) === '.') && lastFirstPartOfPath && lastFirstPartOfPath !== thisFirstPartOfPath) {
                    importsText += '\n';
                }

                importsText += imp.lines.join('\n') + '\n';
                lastFirstPartOfPath = thisFirstPartOfPath;
            });

            newContent = importsText + '\n' + newContent;
            newContent = newContent.replace(/\n{4,}/, '\n\n\n');

            return {
                v: {
                    content: newContent,
                    filesToCreate: newImports.filter(function (imp) {
                        return imp && imp.shouldCreateFile;
                    })
                }
            };
        }();

        if ((typeof _ret2 === 'undefined' ? 'undefined' : _typeof(_ret2)) === "object") return _ret2.v;
    }
}