'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.getConfig = getConfig;

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _season = require('season');

var _season2 = _interopRequireDefault(_season);

var _editorconfig = require('editorconfig');

var _editorconfig2 = _interopRequireDefault(_editorconfig);

var _deepExtend = require('deep-extend');

var _deepExtend2 = _interopRequireDefault(_deepExtend);

var _clone = require('clone');

var _clone2 = _interopRequireDefault(_clone);

var _fs = require('./fs.js');

var fsHelpers = _interopRequireWildcard(_fs);

var _defaultConfig = require('../config/defaultConfig.js');

var _defaultConfig2 = _interopRequireDefault(_defaultConfig);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var CONFIG_FILENAME = '.cerebralrc';

function getConfig(filePath, masterConfig) {
    var userConfig = !masterConfig || masterConfig.useRcFile ? getConfigRc(filePath) : masterConfig;
    var config = (0, _deepExtend2.default)((0, _clone2.default)(_defaultConfig2.default), userConfig);

    // Check editorconfig
    if (config.style.indentationPrefersEditorConfig) {
        var ec = _editorconfig2.default.parseSync(filePath);
        if (ec && ec.indent_style) {
            if (ec.indent_style === 'space') {
                config.style.indentation = new Array(ec.indent_size + 1).join(' ');
            }
            if (ec.indent_style === 'tab') {
                config.style.indentation = '\t';
            }
        }
    }

    config.specialImportsMap = mapSpecialImports(config.specialImports);

    return config;
}

function getConfigRc(filePath) {
    var configPath = fsHelpers.findConfigFilePath(_path2.default.dirname(filePath), CONFIG_FILENAME);

    if (!configPath) {
        return {};
    }

    try {
        var config = _season2.default.readFileSync(configPath);
        return config && config.generator ? config.generator : {};
    } catch (exception) {
        return {};
    }
}

function mapSpecialImports(config) {
    var map = {};
    var processNode = function processNode(node) {
        if (node.keys && node.importPath) {
            node.keys.map(function (key) {
                map[key] = node.importPath.replace(/\{KEY\}/g, key);
            });
        } else {
            Object.keys(node).map(function (key) {
                map[key] = node[key];
            });
        }
    };

    if (config instanceof Array) {
        config.map(processNode);
    } else if (config instanceof Object) {
        processNode(config);
    }

    return map;
}