'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.performOnFile = performOnFile;

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _fs3 = require('./helpers/fs.js');

var fsHelpers = _interopRequireWildcard(_fs3);

var _config = require('./helpers/config.js');

var configHelpers = _interopRequireWildcard(_config);

var _parser = require('./parser.js');

var parser = _interopRequireWildcard(_parser);

var _generator = require('./generator.js');

var generator = _interopRequireWildcard(_generator);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var defaultLogger = {
    log: function log() {}
};

function performOnFile(_ref) {
    var filePath = _ref.filePath;
    var _ref$config = _ref.config;
    var config = _ref$config === undefined ? null : _ref$config;
    var _ref$write = _ref.write;
    var write = _ref$write === undefined ? false : _ref$write;
    var _ref$logger = _ref.logger;
    var logger = _ref$logger === undefined ? defaultLogger : _ref$logger;
    var _ref$content = _ref.content;
    var content = _ref$content === undefined ? false : _ref$content;

    var mergedConfig = configHelpers.getConfig(filePath, config);

    logger.log('CEREBRAL-CLI-GENERATOR: Parsed config', mergedConfig);

    var fileContent = content || _fs2.default.readFileSync(filePath, { encoding: 'utf8' });
    var parseResult = parser.parse(fileContent);

    logger.log('CEREBRAL-CLI-GENERATOR: ParseResult', parseResult);

    if (!parseResult) {
        return false;
    }

    var composed = generator.generate(fileContent, parseResult, mergedConfig);

    logger.log('CEREBRAL-CLI-GENERATOR: composed', composed);

    if (!composed) {
        return false;
    }

    if (write) {
        logger.log('CEREBRAL-CLI-GENERATOR: Generated and wrote imports for ' + filePath);
        _fs2.default.writeFileSync(filePath, composed.content);
    }

    composed.filesToCreate.forEach(function (file) {
        logger.log('CEREBRAL-CLI-GENERATOR: Generated file ' + file.path);
        fsHelpers.createFile(_path2.default.join(_path2.default.dirname(filePath), file.path), file.content);
    });

    return composed.content;
}